Option Compare Binary
Option Explicit On 
Option Strict On

Imports System.IO

''' <summary>
'''   A simple class that can be used to recursively enumerate files and directories in
'''   a given directory.
''' </summary>
Public Class FileSystemEnumerator
    Private m_Stopped As Boolean
    Private m_StartDirectory As String

    ''' <summary>
    '''   Represents the method that will handle the <c>DirectoryFound</c> event of a <c>FileSystemEnumerator</c>.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">A <c>DirectoryFoundEventArgs</c> that contains the event data.</param>
    Public Delegate Sub DirectoryFoundHandler(ByVal sender As Object, ByVal e As DirectoryFoundEventArgs)

    ''' <summary>
    '''   Occurs when a directory is found.
    ''' </summary>
    Public Event DirectoryFound As DirectoryFoundHandler

    ''' <summary>
    '''   Represents the method that will handle the <c>FileFound</c> event of a <c>FileSystemEnumerator</c>.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">A <c>FileFoundEventArgs</c> that contains the event data.</param>
    Public Delegate Sub FileFoundHandler(ByVal sender As Object, ByVal e As FileFoundEventArgs)

    ''' <summary>
    '''   Occurs when a file is found.
    ''' </summary>
    Public Event FileFound As FileFoundHandler

    ''' <summary>
    '''   Represents the method that will handle the <c>EnumerationDirectoryChanged</c> event of a <c>FileSystemEnumerator</c>.
    ''' </summary>
    ''' <param name="sender">The source of the event.</param>
    ''' <param name="e">A <c>EnumerationDirectoryChangedHandler</c> that contains the event data.</param>
    Public Delegate Sub EnumerationDirectoryChangedHandler(ByVal sender As Object, ByVal e As EnumerationDirectoryChangedEventArgs)

    ''' <summary>
    '''   Occurs when the directory to be searched in changed.
    ''' </summary>
    Public Event EnumerationDirectoryChanged As EnumerationDirectoryChangedHandler
    
    ''' <summary>
    '''   Starts the enumeration process.
    ''' </summary>
    Public Sub Start()
        m_Stopped = False
        EnumerateDirectory(m_StartDirectory)
    End Sub

    ''' <summary>
    '''   Stops enumeration.
    ''' </summary>
    Public Sub [Stop]()
        m_Stopped = True
    End Sub

    ''' <summary>
    '''   Gets if the enumeration is currently running.
    ''' </summary>
    ''' <value>A Boolean value indicating of the enumeration is currently running.</value>
    Public ReadOnly Property Running() As Boolean
        Get
            Return Not m_Stopped
        End Get
    End Property

    ''' <summary>
    '''   Gets or sets the directory where the search should be performed.
    ''' </summary>
    ''' <value>The directory where the search should be performed.</value>
    Public Property StartDirectory() As String
        Get
            Return m_StartDirectory
        End Get
        Set(ByVal Value As String)
            m_StartDirectory = Value
        End Set
    End Property

    ''' <summary>
    '''   Raises the <c>DirectoryFound</c> event.
    ''' </summary>
    ''' <param name="e">A <c>DirectoryFoundEventArgs</c> that contains the event data.</param>
    Protected Sub OnDirectoryFound(ByVal e As DirectoryFoundEventArgs)
        SyncLock Me
            RaiseEvent DirectoryFound(Me, e)
        End SyncLock
    End Sub

    ''' <summary>
    '''   Raises the <c>FileFound</c> event.
    ''' </summary>
    ''' <param name="e">A <c>FileFoundEventArgs</c> that contains the event data.</param>
    Protected Sub OnFileFound(ByVal e As FileFoundEventArgs)
        SyncLock Me
            RaiseEvent FileFound(Me, e)
        End SyncLock
    End Sub

    ''' <summary>
    '''   Raises the <c>EnumerationDirectoryChanged</c> event.
    ''' </summary>
    ''' <param name="e">A <c>EnumerationDirectoryChangedEventArgs</c> that contains the event data.</param>
    Protected Sub OnEnumerationDirectoryChanged(ByVal e As EnumerationDirectoryChangedEventArgs)
        SyncLock Me
            RaiseEvent EnumerationDirectoryChanged(Me, e)
        End SyncLock
    End Sub

    ''' <summary>
    '''   Recursively enumerates directories and folders in the directory specified in
    '''   <paramref name="RootDirectory"/>.
    ''' </summary>
    ''' <param name="RootDirectory">Path of the directory where the search should take place.</param>
    Private Sub EnumerateDirectory(ByVal RootDirectory As String)
        SyncLock Me
            OnEnumerationDirectoryChanged(New EnumerationDirectoryChangedEventArgs(RootDirectory))
            For Each s As String In Directory.GetFiles(RootDirectory)
                If m_Stopped Then
                    Return
                Else
                    OnFileFound(New FileFoundEventArgs(s))
                End If
            Next s
            For Each s As String In Directory.GetDirectories(RootDirectory)

                ' Prevent enumeration of reparse points.
                '
                ' Platform SDK: Storage -- Reparse Points
                ' <URL:http://msdn.microsoft.com/library/en-us/fileio/base/reparse_points.asp>
                '
                ' You can create an infinitely recursive directory tree
                ' <URL:http://blogs.msdn.com/oldnewthing/archive/2004/12/27/332704.aspx>
                If Not CBool(File.GetAttributes(s) And FileAttributes.ReparsePoint) Then
                    If m_Stopped Then
                        Return
                    Else
                        Dim e As New DirectoryFoundEventArgs(s)
                        e.CancelEnumeration = False
                        OnDirectoryFound(e)
                        If Not e.CancelEnumeration Then
                            EnumerateDirectory(s)
                        End If
                    End If
                End If
            Next s
        End SyncLock
    End Sub
End Class